/*  aqotdd - Afghan's Quote of the day server - An RFC865 compliant server.
    fortune-tcp,fortune-udp,udpbench,tcpbench - RFC865 compliant clients.

    Copyright (C) 2000 Afghan Hound Software

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/

/* main.h - Includes, defines, function prototypes and global variables for 
   clients and server */

#include <config.h>
#include <stdio.h> 
#include <stdlib.h> 
#include <errno.h>
#include <unistd.h> 
#include <fcntl.h>
#include <signal.h>
#include <string.h> 
#include <ctype.h>
#include <stddef.h>
#include <sys/types.h> 
#include <syslog.h>
#include <stdarg.h>
#include <pwd.h>
#include <netdb.h>
#include <sys/stat.h>
#include <netinet/in.h> 
#include <sys/socket.h> 
#include <arpa/inet.h> 

#if HAVE_DIRENT_H
#include <dirent.h>
#define NAMLEN(dirent) strlen((dirent)->d_name)
#else
#define dirent direct
#define NAMLEN(dirent) (dirent)->d_namlen
#if HAVE_SYS_NDIR_H
#include <sys/ndir.h>
#endif
#if HAVE_SYS_DIR_H
#include <sys/dir.h>
#endif
#if HAVE_NDIR_H
#include <ndir.h>
#endif
#endif


#if TIME_WITH_SYS_TIME
#include <sys/time.h>
#include <time.h>
#else
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif
#endif

#if HAVE_MALLINFO
#ifdef MEMDEBUG
#include <malloc.h>
#endif
#else
#undef MEMDEBUG
#endif

#if HAVE_SYSLOG_H
/*Nothing*/
#else
#define DEF_SYSLOG 0
#endif

/* Random globals */
int   debug_flag;

int   got_alarm;

/* Client */
char *command_port;
char *command_host;

/* Config Variables */
char *tcp_port,*udp_port;
int   use_tcp,  use_udp;
int   use_off,  use_rfc;
int   max_con_udp;
int   max_con_tcp;
int   daemon_flag;
int   mark_time,interval_time;
int   wait_flag;
int   use_sys;
int   log_con,log_msg,chk_con;
int   log_count;
int   abs_max,abs_min;
int   order;
char *allow,*deny;
char *qotd_dir,*def_user,*log_file;
FILE *log_fileh;


/* Network Variables */
int    real_tcp,real_udp;
int    real_tcp_port,real_udp_port;
fd_set sockets;
int    listen_fd_tcp, listen_fd_udp, highsocket;
int    tcp_connects, udp_connects;



/* Debug / logging stuff */
int udp_count, tcp_count, realloc1, realloc2;

/* Network Stuff */
void  net_read_sock();
void  net_init();
void  net_build_control_lists();

/* Signal handlers */
RETSIGTYPE  signal_check();
RETSIGTYPE  signal_init();

/* Read config file */
void  config_readfile();
void  config_findfile();
void  config_parseoption(int c,char **v);

/* Quotes */
void  qotd_init();
char *qotd_getquote(char *buffer);

/* Logs */
void  log_message(int level,char *format,...);
void  log_printready();
void  my_message(int type,char *msg1,char *msg2);

/* Common */
int   atoport(char *service,char *proto,int type);

/* For clients */
char *get_host(char *buf);
int   get_port(char *proto);
int   parse_client_option(int c,char **v,char *which);
void  parse_bench_option(int c,char **v,char *which);
void check_bence_options();
struct in_addr *atoaddr(char *address);

/* Init other stuff */
void  first_init(int c,char **a);
void  daemon_init();

/* Do things */
void *xmalloc (size_t size);
void *xrealloc(void *original,size_t new_size);
